# Copyright 2019 Regents of the University of Minnesota.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tutorial hello world processor."""
import mtap


@mtap.processor('hello')
class HelloProcessor(mtap.DocumentProcessor):
    def process_document(self, document, params):
        with document.get_labeler('hello') as add_hello:
            text = document.text
            add_hello(0, len(text), response='Hello ' + text + '!')


if __name__ == '__main__':
    mtap.run_processor(HelloProcessor())



#  Copyright 2020 Regents of the University of Minnesota.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

import pytest

from mtap import GenericLabel, Location, Document
from mtap.data._label_indices import presorted_label_index


document = Document('plaintext', text='blah')


@pytest.fixture
def tested():
    return presorted_label_index([
        GenericLabel(0, 5, document=document, i=7),
        GenericLabel(0, 7, document=document, i=6),
        GenericLabel(2, 6, document=document, i=5),
        GenericLabel(6, 7, document=document, i=4),
        GenericLabel(6, 8, document=document, i=3),
        GenericLabel(9, 10, document=document, i=2),
        GenericLabel(9, 13, document=document, i=1),
        GenericLabel(9, 13, document=document, i=0),
    ]).descending()


def test_getitem(tested):
    assert tested[3] == GenericLabel(6, 8, document=document, i=3)


def test_getitem_first(tested):
    assert tested[0] == GenericLabel(9, 13, document=document, i=0)


def test_getitem_last(tested):
    assert tested[7] == GenericLabel(0, 5, document=document, i=7)


def test_getitem_negative(tested):
    assert tested[-4] == GenericLabel(6, 7, document=document, i=4)


def test_getitem_last_negative(tested):
    assert tested[-1] == GenericLabel(0, 5, document=document, i=7)


def test_getitem_slice(tested):
    sliced = tested[2:4]
    assert sliced == [
        GenericLabel(9, 10, document=document, i=2),
        GenericLabel(6, 8, document=document, i=3),
    ]


def test_getitem_slice_end(tested):
    assert tested[4:8] == [
        GenericLabel(6, 7, document=document, i=4),
        GenericLabel(2, 6, document=document, i=5),
        GenericLabel(0, 7, document=document, i=6),
        GenericLabel(0, 5, document=document, i=7),
    ]


def test_getitem_slice_open_left(tested):
    assert tested[:4] == [
        GenericLabel(9, 13, document=document, i=0),
        GenericLabel(9, 13, document=document, i=1),
        GenericLabel(9, 10, document=document, i=2),
        GenericLabel(6, 8, document=document, i=3),
    ]


def test_getitem_slice_open_right(tested):
    assert tested[4:] == [
        GenericLabel(6, 7, document=document, i=4),
        GenericLabel(2, 6, document=document, i=5),
        GenericLabel(0, 7, document=document, i=6),
        GenericLabel(0, 5, document=document, i=7),
    ]


def test_getitem_slice_neg_right(tested):
    assert tested[4:-1] == [
        GenericLabel(6, 7, document=document, i=4),
        GenericLabel(2, 6, document=document, i=5),
        GenericLabel(0, 7, document=document, i=6),
    ]


def test_getitem_slice_neg_left(tested):
    assert tested[-4:-1] == [
        GenericLabel(6, 7, document=document, i=4),
        GenericLabel(2, 6, document=document, i=5),
        GenericLabel(0, 7, document=document, i=6),
    ]


def test_getitem_not_idx_slice(tested):
    with pytest.raises(TypeError):
        tested['foo']


def tested_getitem_slice_step_not_one(tested):
    slice = tested[1:4:2]
    assert slice == ([
        GenericLabel(9, 13, document=document, i=1),
        GenericLabel(6, 8, document=document, i=3),
    ])


def test_at(tested):
    assert tested.at(GenericLabel(2, 6, document=document))[0] == GenericLabel(2, 6, document=document, i=5)


def test_at_location(tested):
    assert tested.at(Location(2, 6))[0] == GenericLabel(2, 6, document=document, i=5)


def test_at_location_multiple(tested):
    assert tested.at(Location(9, 13)) == [
        GenericLabel(9, 13, document=document, i=0),
        GenericLabel(9, 13, document=document, i=1),
    ]


def test_at_location_not_found(tested):
    assert tested.at(Location(10, 10)) == []


def test_len(tested):
    assert len(tested) == 8


def test_covering(tested):
    covering = tested.covering(2, 4)
    assert list(covering) == [
        GenericLabel(2, 6, document=document, i=5),
        GenericLabel(0, 7, document=document, i=6),
        GenericLabel(0, 5, document=document, i=7),
    ]


def test_covering_empty(tested):
    assert tested.covering(4, 10) == []


def test_empty_covering(tested):
    covering = tested.covering(4, 10)
    assert list(covering) == []


def test_inside(tested):
    inside = tested.inside(1, 8)
    assert list(inside) == [
        GenericLabel(6, 8, document=document, i=3),
        GenericLabel(6, 7, document=document, i=4),
        GenericLabel(2, 6, document=document, i=5),
    ]


def test_inside_before(tested):
    inside = tested.inside(0, 3)
    assert list(inside) == []


def test_inside_after(tested):
    inside = tested.inside(15, 20)
    assert list(inside) == []


def test_inside_many(tested):
    tested = presorted_label_index([
        GenericLabel(0, 3, document=document),
        GenericLabel(0, 3, document=document),
        GenericLabel(0, 3, document=document),
        GenericLabel(0, 3, document=document),
        GenericLabel(0, 3, document=document),
        GenericLabel(0, 3, document=document),
        GenericLabel(0, 3, document=document),
        GenericLabel(2, 5, document=document),
        GenericLabel(2, 5, document=document),
        GenericLabel(2, 5, document=document),
        GenericLabel(2, 5, document=document),
        GenericLabel(2, 5, document=document),
        GenericLabel(2, 5, document=document),
        GenericLabel(2, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(6, 6, document=document),
        GenericLabel(6, 6, document=document),
        GenericLabel(6, 6, document=document),
        GenericLabel(6, 6, document=document),
        GenericLabel(6, 6, document=document),
        GenericLabel(6, 6, document=document),
        GenericLabel(6, 6, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
        GenericLabel(6, 10, document=document),
    ])
    inside = tested.inside(3, 6)
    assert inside == [
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(3, 5, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
    ]
    inside = inside.inside(5, 6)
    assert inside == [
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
        GenericLabel(5, 6, document=document),
    ]


def test_begins_inside(tested):
    inside = tested.beginning_inside(1, 9)
    assert list(inside) == [
        GenericLabel(6, 8, document=document, i=3),
        GenericLabel(6, 7, document=document, i=4),
        GenericLabel(2, 6, document=document, i=5),
    ]


def test_begins_inside_empty(tested):
    inside = tested.beginning_inside(3, 5)
    assert inside == []


def test_ascending(tested):
    ascending = tested.ascending()
    assert ascending == [
        GenericLabel(0, 5, document=document, i=7),
        GenericLabel(0, 7, document=document, i=6),
        GenericLabel(2, 6, document=document, i=5),
        GenericLabel(6, 7, document=document, i=4),
        GenericLabel(6, 8, document=document, i=3),
        GenericLabel(9, 10, document=document, i=2),
        GenericLabel(9, 13, document=document, i=1),
        GenericLabel(9, 13, document=document, i=0),
    ]


def test_descending(tested):
    descending = tested.descending()
    assert descending == tested


def test_before(tested):
    before = tested.before(8)
    assert before == [
        GenericLabel(6, 8, document=document, i=3),
        GenericLabel(6, 7, document=document, i=4),
        GenericLabel(2, 6, document=document, i=5),
        GenericLabel(0, 7, document=document, i=6),
        GenericLabel(0, 5, document=document, i=7),
    ]


def test_before_start(tested):
    before = tested.before(3)
    assert before == []


def test_after(tested):
    after = tested.after(2)
    assert after == [
        GenericLabel(9, 13, document=document, i=0),
        GenericLabel(9, 13, document=document, i=1),
        GenericLabel(9, 10, document=document, i=2),
        GenericLabel(6, 8, document=document, i=3),
        GenericLabel(6, 7, document=document, i=4),
        GenericLabel(2, 6, document=document, i=5),
    ]


def test_contains_true(tested):
    assert GenericLabel(9, 13, document=document, i=0) in tested


def test_contains_false_location_in(tested):
    assert GenericLabel(9, 13, document=document) not in tested


def test_contains_false_location_not_in(tested):
    assert GenericLabel(0, 4, document=document) not in tested


def test_contains_false_not_label(tested):
    assert "blub" not in tested


def test_reversed(tested):
    l = list(reversed(tested))
    assert l == [
        GenericLabel(0, 5, document=document, i=7),
        GenericLabel(0, 7, document=document, i=6),
        GenericLabel(2, 6, document=document, i=5),
        GenericLabel(6, 7, document=document, i=4),
        GenericLabel(6, 8, document=document, i=3),
        GenericLabel(9, 10, document=document, i=2),
        GenericLabel(9, 13, document=document, i=1),
        GenericLabel(9, 13, document=document, i=0),
    ]


def test_count_in(tested):
    assert tested.count(GenericLabel(2, 6, document=document, i=5)) == 1


def test_count_multiple(tested):
    index = presorted_label_index([
        GenericLabel(2, 6, document=document, i=2),
        GenericLabel(6, 7, document=document, i=3),
        GenericLabel(6, 8, document=document, i=4),
        GenericLabel(9, 10, document=document, i=5),
        GenericLabel(9, 13, document=document, i=6),
        GenericLabel(9, 13, document=document, i=7),
        GenericLabel(9, 13, document=document, i=6)
    ]).descending()
    assert index.count(GenericLabel(9, 13, document=document, i=6)) == 2


def test_count_different_label(tested):
    assert tested.count(GenericLabel(9, 13, document=document, x=2)) == 0


def test_count_not_label(tested):
    assert tested.count("blub") == 0


def test_count_location_not_in(tested):
    assert tested.count(GenericLabel(4, 5, document=document)) == 0


def test_filter(tested):
    assert tested.filter(lambda x: x.i % 2 == 0) == [
        GenericLabel(9, 13, document=document, i=0),
        GenericLabel(9, 10, document=document, i=2),
        GenericLabel(6, 7, document=document, i=4),
        GenericLabel(0, 7, document=document, i=6),
    ]



# Copyright 2019 Regents of the University of Minnesota.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Hello world tutorial pipeline."""
import sys

if __name__ == '__main__':
    from mtap import Document, Event, EventsClient, Pipeline, RemoteProcessor

    with EventsClient(address=sys.argv[1]) as client, \
            Pipeline(
                RemoteProcessor(processor_id='hello', address=sys.argv[2])
            ) as pipeline:
        with Event(event_id='1', client=client) as event:
            document = Document(document_name='name', text='YOUR NAME')
            event.add_document(document)
            pipeline.run(document)
            index = document.get_label_index('hello')
            for label in index:
                print(label.response)



# Copyright 2019 Regents of the University of Minnesota.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import signal
import time
from pathlib import Path
from subprocess import Popen, TimeoutExpired, PIPE, STDOUT

import pytest
import requests
from requests import RequestException

import mtap
from mtap import RemoteProcessor, EventsClient, Event
from mtap.utilities import subprocess_events_server, find_free_port


@pytest.fixture(name='disc_python_events')
def fixture_disc_python_events():
    with subprocess_events_server(cwd=Path(__file__).parents[2], register=True) as address:
        yield address


@pytest.fixture(name='disc_python_processor')
def fixture_disc_python_processor(disc_python_events, processor_watcher):
    p = Popen(['python', '-m', 'mtap.examples.example_processor',
               '-p', '50501', '--register'],
              start_new_session=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT)
    yield from processor_watcher(address="127.0.0.1:50501", process=p)


@pytest.fixture(name="disc_java_processor")
def fixture_disc_java_processor(disc_python_events, processor_watcher):
    mtap_jar = os.environ['MTAP_JAR']
    p = Popen(['java', '-cp', mtap_jar,
               'edu.umn.nlpie.mtap.examples.WordOccurrencesExampleProcessor',
               '-p', '50502', '--register'],
              stdin=PIPE, stdout=PIPE, stderr=STDOUT)
    yield from processor_watcher(address="127.0.0.1:50502", process=p)


@pytest.fixture(name="disc_api_gateway")
def fixture_disc_api_gateway(disc_python_events, disc_python_processor, disc_java_processor):
    p = Popen(['mtap-gateway', '-logtostderr', '-v=3'], stdin=PIPE, stdout=PIPE, stderr=STDOUT)
    session = requests.Session()
    session.trust_env = False
    try:
        if p.returncode is not None:
            raise ValueError("Failed to launch go gateway")
        for i in range(6):
            if i == 5:
                raise ValueError("Failed to connect to go gateway")
            try:
                time.sleep(3)
                resp = session.get("http://localhost:50503/v1/processors")
                if resp.status_code == 200 and len(resp.json()['Processors']) == 2:
                    break
            except RequestException:
                pass
        yield
    finally:
        session.close()
        p.send_signal(signal.SIGINT)
        try:
            stdout, _ = p.communicate(timeout=1)
            print("api gateway exited with code: ", p.returncode)
            print(stdout.decode('utf-8'))
        except TimeoutExpired:
            print("timed out waiting for api gateway to terminate")


PHASERS = """
Maybe if we felt any human loss as keenly as we feel one of those close to us, human history would 
be far less bloody. The Enterprise computer system is controlled by three primary main processor 
cores, cross-linked with a redundant melacortz ramistat, fourteen kiloquad interface modules. Our 
neural pathways have become accustomed to your sensory input patterns. Mr. Worf, you do remember 
how to fire phasers?"""


@pytest.mark.consul
def test_disc_pipeline(disc_python_events, disc_python_processor, disc_java_processor):
    with EventsClient(address=disc_python_events) as client, mtap.Pipeline(
            RemoteProcessor('mtap-example-processor-python', address='localhost:50501',
                            params={'do_work': True}),
            RemoteProcessor('mtap-example-processor-java', address='localhost:50502',
                            params={'do_work': True})
    ) as pipeline:
        with Event(event_id='1', client=client) as event:
            event.metadata['a'] = 'b'
            document = event.create_document('plaintext', PHASERS)
            pipeline.run(document)
            letter_counts = document.get_label_index('mtap.examples.letter_counts')
            a_counts = letter_counts[0]
            assert a_counts.count == 23
            b_counts = letter_counts[1]
            assert b_counts.count == 6
            pipeline.print_times()
            thes = document.get_label_index("mtap.examples.word_occurrences")
            assert thes[0].start_index == 121
            assert thes[0].end_index == 124


@pytest.mark.consul
def test_disc_api_gateway(disc_api_gateway):
    session = requests.Session()
    session.trust_env = False
    resp = session.get("http://localhost:50503/v1/processors")
    assert resp.status_code == 200
    processors = resp.json()
    all_ids = []
    for processor in processors['Processors']:
        all_ids.append(processor['Identifier'])
    assert 'mtap-example-processor-python' in all_ids
    assert 'mtap-example-processor-java' in all_ids



# Copyright 2019 Regents of the University of Minnesota.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from mtap import Event, EventsClient, Document, GenericLabel
from mtap.data import DISTINCT_GENERIC_ADAPTER, GENERIC_ADAPTER


def test_add_labels_not_distinct(mocker):
    client = mocker.Mock(EventsClient)
    client.get_local_instance.return_value = client
    client.get_label_index_info.return_value = []
    event = Event(event_id='1', client=client)
    document = Document(document_name='plaintext',
                        text='The quick brown fox jumped over the lazy dog.', event=event)
    labels = [
        GenericLabel(0, 10, document=document, x=1),
        GenericLabel(11, 15, document=document, x=2),
        GenericLabel(16, 20, document=document, x=3)
    ]
    document.add_labels('index', labels)
    client.add_labels.assert_called_with(
        event_id='1',
        document_name='plaintext',
        index_name='index',
        labels=labels,
        adapter=mocker.ANY
    )
    l2 = document.labels['index']
    assert l2 == labels
    assert not l2.distinct


def test_add_labels_distinct(mocker):
    client = mocker.Mock(EventsClient)
    client.get_local_instance.return_value = client
    client.get_label_index_info.return_value = []
    event = Event(event_id='1', client=client)
    document = Document(document_name='plaintext',
                        text='The quick brown fox jumped over the lazy dog.', event=event)
    labels = [
        GenericLabel(0, 10, document=document, x=1),
        GenericLabel(11, 15, document=document, x=2),
        GenericLabel(16, 20, document=document, x=3)
    ]
    document.add_labels('index', labels, distinct=True)
    client.add_labels.assert_called_with(
        event_id='1',
        document_name='plaintext',
        index_name='index',
        labels=labels,
        adapter=mocker.ANY
    )
    l2 = document.labels['index']
    assert l2 == labels
    assert l2.distinct


def test_labeler_not_distinct_default(mocker):
    client = mocker.Mock(EventsClient)
    client.get_local_instance.return_value = client
    client.get_label_index_info.return_value = []
    event = Event(event_id='1', client=client)
    document = Document(document_name='plaintext',
                        text='The quick brown fox jumped over the lazy dog.', event=event)
    with document.get_labeler('index') as add_generic_label:
        add_generic_label(0, 10, x=1)
        add_generic_label(11, 15, x=2)
        add_generic_label(16, 20, x=3)
    labels = [
        GenericLabel(0, 10, document=document, x=1),
        GenericLabel(11, 15, document=document, x=2),
        GenericLabel(16, 20, document=document, x=3)
    ]
    label_adapter = GENERIC_ADAPTER
    client.add_labels.assert_called_with(
        event_id='1',
        document_name='plaintext',
        index_name='index',
        labels=labels,
        adapter=label_adapter
    )
    assert document.labels['index'] == labels


def test_labeler_distinct(mocker):
    client = mocker.Mock(EventsClient)
    client.get_local_instance.return_value = client
    client.get_label_index_info.return_value = []
    event = Event(event_id='1', client=client)
    document = Document(document_name='plaintext',
                        text='The quick brown fox jumped over the lazy dog.', event=event)
    with document.get_labeler('index', distinct=True) as add_generic_label:
        add_generic_label(0, 10, x=1)
        add_generic_label(11, 15, x=2)
        add_generic_label(16, 20, x=3)
    labels = [
        GenericLabel(0, 10, document=document, x=1),
        GenericLabel(11, 15, document=document, x=2),
        GenericLabel(16, 20, document=document, x=3)
    ]
    label_adapter = DISTINCT_GENERIC_ADAPTER
    client.add_labels.assert_called_with(
        event_id='1',
        document_name='plaintext',
        index_name='index',
        labels=labels,
        adapter=label_adapter
    )
    assert document.labels['index'] == labels



#  Copyright 2019 Regents of the University of Minnesota.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
from tempfile import TemporaryFile

try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

from mtap import Event, Document, label
from mtap.io.serialization import YamlSerializer


def test_yml_serializer():
    event = Event(event_id='1')
    event.metadata['foo'] = "bar"
    document = Document('plaintext', text='Some text.')
    event.add_document(document)
    one = label(start_index=0, end_index=5, x=10)
    two = label(start_index=6, end_index=10, x=15)
    document.add_labels('one', [one,
                                two])
    document.add_labels('two', [label(start_index=0, end_index=25, a='b', b=one),
                                label(start_index=26, end_index=42, a='c', b=two)])
    document.add_labels('three', [
        label(start_index=0, end_index=10, foo=True),
        label(start_index=11, end_index=15, foo=False)
    ], distinct=True)

    with TemporaryFile('w+') as tf:
        YamlSerializer.event_to_file(event, tf)
        tf.flush()
        tf.seek(0)
        e = YamlSerializer.file_to_event(tf)

    assert e.event_id == event.event_id
    assert e.metadata['foo'] == 'bar'
    d = e.documents['plaintext']
    assert d.text == document.text
    index_one = d.labels['one']
    assert index_one == [one, two]
    index_two = d.labels['two']
    assert index_two == [label(start_index=0, end_index=25, a='b', b=one),
                         label(start_index=26, end_index=42, a='c', b=two)]
    index_three = d.labels['three']
    assert index_three == [label(start_index=0, end_index=10, foo=True),
                           label(start_index=11, end_index=15, foo=False)]



#  Copyright 2020 Regents of the University of Minnesota.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
"""Internal implementation of a standard non-distinct label index."""
from abc import ABCMeta, abstractmethod
from bisect import bisect_left, bisect_right
from operator import attrgetter
from typing import Union, Optional, Any, Iterator, List, TypeVar, Sequence, Generic, Callable, \
    TYPE_CHECKING

from mtap.data import _labels

if TYPE_CHECKING:
    from mtap import data

L = TypeVar('L', bound='data.Label')


class LabelIndex(Sequence[L], Generic[L]):
    """An immutable :obj:`~typing.Sequence` of labels ordered by their location in text. By default
    sorts by ascending `start_index` and then by ascending `end_index`.
    """
    __slots__ = ()

    @property
    @abstractmethod
    def distinct(self) -> bool:
        """bool: Whether this label index is distinct, i.e. all of the labels in it are
            non-overlapping.
        """
        ...

    @property
    @abstractmethod
    def adapter(self) -> Optional['data.ProtoLabelAdapter']:
        ...

    @abstractmethod
    def filter(self, fn: Callable[['data.Label'], bool]) -> 'data.LabelIndex[L]':
        """Filters the label index according to a filter function.

        This function is less efficient for filtering based on indices than
        :func:`~LabelIndex.inside`, :func:`~LabelIndex.covering`, etc., which use a binary search
        method on the sorted index.

        Args:
            fn (~typing.Callable[[Label], bool]): A filter function, returns ``true`` if the label
                should be included, ``false`` if it should not be included

        Returns:
            LabelIndex: A view of this label index.
        """
        ...

    @abstractmethod
    def at(self,
           x: Union['data.Label', 'data.Location', float],
           end: Optional[float] = None) -> 'data.LabelIndex[L]':
        """Returns the labels at the specified location in text.

        Args:
            x (~typing.Union[Label, Location, float]): A label or location, or start index if `end`
                is specified.
            end (~typing.Optional[float]): The exclusive end index of the location in text if it has
                not been specified by a label.

        Returns:
            LabelIndex: A view of this label index.

        Examples:
            >>> from mtap import label_index, label
            >>> index = label_index([label(0, 10, x=1),
            ...                      label(0, 10, x=2),
            ...                      label(6, 20, x=3)])
            >>> index.at(0, 10)
            label_index([GenericLabel(0, 10, x=1), GenericLabel(0, 10, x=2)], distinct=False)
        """
        ...

    @abstractmethod
    def covering(self,
                 x: Union['data.Label', 'data.Location', float],
                 end: Optional[float] = None) -> 'data.LabelIndex[L]':
        """A label index containing all labels that cover / contain the specified location in text.

        Args:
            x (~typing.Union[~'data.Label', ~'data.Location', float]): A label or location, or start
                index if `end` is specified.
            end (~typing.Optional[float]): The exclusive end index of the location in text if it has
                not been specified by a label.

        Returns:
            LabelIndex: A view of this label index.

        Examples:
            >>> from mtap import label_index, label
            >>> index = label_index([label(0, 5, x=1),
            ...                      label(0, 10, x=2),
            ...                      label(5, 10, x=3),
            ...                      label(7, 10, x=4),
            ...                      label(5, 15, x=5),
            ...                      label(10, 15, x=6)])
            >>> index.covering(5, 10)
            label_index([GenericLabel(0, 10, x=2), GenericLabel(5, 10, x=3), GenericLabel(5, 15, x=5)], distinct=False)

        """
        ...

    @abstractmethod
    def inside(self,
               x: Union['data.Label', 'data.Location', float],
               end: Optional[float] = None) -> 'data.LabelIndex[L]':
        """A label index containing all labels that are inside the specified location in text.

        Args:
            x (~typing.Union[~'data.Label', ~'data.Location', float]): A label or location, or start
                index if `end` is specified.
            end (~typing.Optional[float]): The exclusive end index of the location in text if it has
                not been specified by a label.

        Returns:
            LabelIndex: A view of this label index.

        Examples:
            >>> from mtap import label_index, label
            >>> index = label_index([label(0, 5, x=1),
            ...                      label(0, 10, x=2),
            ...                      label(5, 10, x=3),
            ...                      label(7, 10, x=4),
            ...                      label(5, 15, x=5),
            ...                      label(10, 15, x=6)])
            >>> index.inside(5, 10)
            label_index([GenericLabel(5, 10, x=3), GenericLabel(7, 10, x=4)], distinct=False)
        """
        ...

    @abstractmethod
    def beginning_inside(self,
                         x: Union['data.Label', 'data.Location', float],
                         end: Optional[float] = None) -> 'data.LabelIndex[L]':
        """A label index containing all labels whose begin index is inside the specified location
        in text.

        Args:
            x (~typing.Union[~'data.Label', ~'data.Location', float]): A label or location, or start
                index if `end` is specified.
            end (~typing.Optional[float]): The exclusive end index of the location in text if it has
                not been specified by a label.

        Returns:
            LabelIndex: A view of this label index.

        Examples:
            >>> from mtap import label_index, label
            >>> index = label_index([label(0, 5, x=1),
            ...                      label(0, 10, x=2),
            ...                      label(5, 10, x=3),
            ...                      label(7, 10, x=4),
            ...                      label(5, 15, x=5),
            ...                      label(10, 15, x=6)])
            >>> index.beginning_inside(6, 11)
            label_index([GenericLabel(7, 10, x=4), GenericLabel(10, 15, x=6)], distinct=False)
        """
        ...

    @abstractmethod
    def overlapping(self,
                    x: Union['data.Label', 'data.Location', float],
                    end: Optional[float] = None) -> 'data.LabelIndex[L]':
        """Returns all labels that overlap the specified location in text.

        Args:
            x (~typing.Union[~'data.Label', ~'data.Location', float]): A label or location, or start
                index if `end` is specified.
            end (~typing.Optional[float]): The exclusive end index of the location in text if it has
                not been specified by a label.

        Returns:
            LabelIndex: A view of this label index.

        Examples:
            >>> from mtap import label_index, label
            >>> index = label_index([label(0, 5, x=1),
            ...                      label(0, 10, x=2),
            ...                      label(5, 10, x=3),
            ...                      label(7, 10, x=4),
            ...                      label(5, 15, x=5),
            ...                      label(10, 15, x=6)])
            >>> index.overlapping(6, 10)
            label_index([GenericLabel(0, 10, x=2), GenericLabel(5, 10, x=3), GenericLabel(5, 15, x=5), GenericLabel(7, 10, x=4)], distinct=False)


        """
        ...

    def before(self, x: Union['data.Label', 'data.Location', float]) -> 'data.LabelIndex[L]':
        """A label index containing all labels that are before a label's location in text or
        an index in text.

        Args:
            x (~typing.Union[~'data.Label', ~'data.Location', float]): A label or location whose
                `start_index` will be used, or a float index in text.

        Returns:
            LabelIndex: A view of this label index.

        Examples:
            >>> from mtap import label_index, label
            >>> index = label_index([label(0, 5, x=1),
            ...                      label(0, 10, x=2),
            ...                      label(5, 10, x=3),
            ...                      label(7, 10, x=4),
            ...                      label(5, 15, x=5),
            ...                      label(10, 15, x=6)])
            >>> index.before(6)
            label_index([GenericLabel(0, 5, x=1)], distinct=False)
        """
        try:
            index = x.start_index
        except AttributeError:
            index = x
        return self.inside(0, index)

    def after(self, x: Union['data.Label', 'data.Location', float]) -> 'data.LabelIndex[L]':
        """A label index containing all labels that are after a label's location in text
        or an index in text.

        Args:
            x (~typing.Union[Label, Location, float]): A label or location whose `end_index` will
                be used, or a float index in text.

        Returns:
            LabelIndex: A view of this label index.

        Examples:
            >>> from mtap import label_index, label
            >>> index = label_index([label(0, 5, x=1),
            ...                      label(0, 10, x=2),
            ...                      label(5, 10, x=3),
            ...                      label(7, 10, x=4),
            ...                      label(5, 15, x=5),
            ...                      label(10, 15, x=6)])
            >>> index.after(6)
            label_index([GenericLabel(7, 10, x=4), GenericLabel(10, 15, x=6)], distinct=False)

        """
        try:
            index = x.end_index
        except AttributeError:
            index = x
        return self.inside(index, float('inf'))

    @abstractmethod
    def ascending(self) -> 'data.LabelIndex[L]':
        """This label index sorted according to ascending start and end index.

        Returns:
            LabelIndex: A view of this label index.

        Examples:
            >>> from mtap import label_index, label
            >>> index = label_index([label(0, 5, x=1),
            ...                      label(0, 10, x=2),
            ...                      label(5, 10, x=3),
            ...                      label(7, 10, x=4),
            ...                      label(5, 15, x=5),
            ...                      label(10, 15, x=6)])
            >>> index == index.ascending()
            True
        """
        ...

    @abstractmethod
    def descending(self) -> 'data.LabelIndex[L]':
        """This label index sorted according to descending start index and ascending end index.

        Returns:
            LabelIndex: A view of this label index.

        Examples:
            >>> from mtap import label_index, label
            >>> index = label_index([label(0, 5, x=1),
            ...                      label(0, 10, x=2),
            ...                      label(5, 10, x=3),
            ...                      label(7, 10, x=4),
            ...                      label(5, 15, x=5),
            ...                      label(10, 15, x=6)])
            >>> index.descending()
            label_index([GenericLabel(10, 15, x=6), GenericLabel(7, 10, x=4), GenericLabel(5, 15, x=5), GenericLabel(5, 10, x=3), GenericLabel(0, 10, x=2), GenericLabel(0, 5, x=1)], distinct=False)
        """
        ...


def label_index(labels: List[L],
                distinct: bool = False,
                adapter: Optional['data.ProtoLabelAdapter'] = None) -> LabelIndex[L]:
    """Creates a label index from labels.

    Args:
        labels (~typing.List[L]): Zero or more labels to create a label index from.
        distinct (bool): Whether the label index is distinct or not.
        adapter (~data.ProtoLabelAdapter): The label adapter for these labels.

    Returns:
        ~data.LabelIndex: The newly created label index.

    Examples:
        >>> from mtap import label_index, label
        >>> index = label_index([label(0, 5, x=1),
        ...                      label(0, 10, x=2),
        ...                      label(5, 10, x=3),
        ...                      label(7, 10, x=4),
        ...                      label(5, 15, x=5),
        ...                      label(10, 15, x=6)])
        >>> index
        label_index([GenericLabel(0, 5, x=1), GenericLabel(0, 10, x=2), GenericLabel(5, 10, x=3), GenericLabel(5, 15, x=5), GenericLabel(7, 10, x=4), GenericLabel(10, 15, x=6)], distinct=False)

    """
    labels = sorted(labels, key=attrgetter('location'))
    return presorted_label_index(labels, distinct, adapter=adapter)


def presorted_label_index(labels: List[L],
                          distinct: bool = False,
                          adapter: Optional['data.ProtoLabelAdapter'] = None) -> LabelIndex[L]:
    if len(labels) == 0:
        return _Empty(distinct, adapter)
    return _LabelIndex(distinct, _Ascending(labels), adapter=adapter)


class _View(metaclass=ABCMeta):
    __slots__ = ('_labels', '_indices', '_locations')

    def __init__(self, labels, indices: Sequence[int] = ...):
        self._labels = labels
        self._indices = range(len(labels)) if indices is ... else indices

    @property
    def locations(self):
        try:
            locations = self._locations
        except AttributeError:
            locations = [self._labels[i].location for i in self._indices]
            self._locations = locations
        return locations

    def __len__(self):
        return len(self._indices)

    def bound(self,
              min_start: float = 0,
              max_start: float = float('inf'),
              min_end: float = 0,
              max_end: float = float('inf')) -> '_View':
        filtered_indices = list(self._filter(min_start, max_start, min_end, max_end))
        if len(filtered_indices) == 0:
            raise ValueError
        return self.__class__(self._labels, filtered_indices)

    def filter(self, fn: Callable[['data.Label'], bool]) -> '_View':
        filtered_indices = list(self._filter_fn(fn))
        if len(filtered_indices) == 0:
            raise ValueError
        return self.__class__(self._labels, filtered_indices)

    def _filter(self,
                min_start: float = 0,
                max_start: float = float('inf'),
                min_end: float = 0,
                max_end: float = float('inf')) -> Iterator[int]:
        try:
            left = self._index_lt(_labels.Location(min_start, min_end)) + 1
        except ValueError:
            left = 0

        for i in range(left, len(self._indices)):
            location = self.locations[i]
            if location.start_index > max_start:
                break
            if min_end <= location.end_index <= max_end:
                yield self._indices[i]

    def _filter_fn(self, fn: Callable[['data.Label'], bool]) -> Iterator[int]:
        for index in self._indices:
            label = self._labels[index]
            if fn(label):
                yield index

    def _first_index_location(self,
                              location: 'data.Location',
                              from_index: int = 0,
                              to_index: int = ...) -> int:
        if to_index is ...:
            to_index = len(self.locations)
        i = bisect_left(self.locations, location, from_index, to_index)

        if i != to_index and self.locations[i] == location:
            return i
        raise ValueError

    def _last_index_location(self,
                             location: 'data.Location',
                             from_index: int = 0,
                             to_index: int = ...):
        if to_index is ...:
            to_index = len(self.locations)
        i = bisect_right(self.locations, location, from_index, to_index)

        if i > from_index and self.locations[i - 1] == location:
            return i - 1
        raise ValueError

    def _index_lt(self,
                  location: 'data.Location',
                  from_index: int = 0,
                  to_index: int = ...) -> int:
        if to_index is ...:
            to_index = len(self.locations)

        i = bisect_left(self.locations, location, from_index, to_index)
        if i > from_index:
            return i - 1
        raise ValueError

    @abstractmethod
    def __getitem__(self, idx) -> Union['data.Label', '_View']:
        ...

    @abstractmethod
    def __iter__(self) -> Iterator['data.Label']:
        ...

    @abstractmethod
    def reversed(self):
        ...

    @abstractmethod
    def index_location(self,
                       location: 'data.Location',
                       from_index: int = 0,
                       to_index: int = ...) -> int:
        ...


class _Ascending(_View):
    __slots__ = ()

    def __getitem__(self, idx) -> Union['data.Label', '_View']:
        if isinstance(idx, int):
            return self._labels[self._indices[idx]]
        elif isinstance(idx, slice):
            return _Ascending(self._labels, self._indices[idx])

    def __iter__(self) -> Iterator['data.Label']:
        for idx in self._indices:
            yield self._labels[idx]

    def reversed(self):
        return _Descending(self._labels, self._indices)

    def index_location(self,
                       location: 'data.Location',
                       from_index: int = 0,
                       to_index: int = ...) -> int:
        return self._first_index_location(location, from_index, to_index)


class _Descending(_View):
    __slots__ = ()

    def _reverse_index(self, i: int = None):
        return -(i + 1)

    def __getitem__(self, idx) -> Union['data.Label', '_View']:
        if isinstance(idx, int):
            return self._labels[self._indices[self._reverse_index(idx)]]
        elif isinstance(idx, slice):
            start = idx.stop
            if start is not None:
                start = self._reverse_index(idx.stop - 1)
            stop = idx.start
            if stop is not None:
                stop = self._reverse_index(idx.start - 1)
            idx = slice(start, stop, idx.step)
            return _Descending(self._labels, self._indices[idx])

    def __iter__(self) -> Iterator['data.Label']:
        for i in range(len(self._indices)):
            yield self._labels[self._indices[self._reverse_index(i)]]

    def reversed(self):
        return _Ascending(self._labels, self._indices)

    def index_location(self,
                       location: 'data.Location',
                       from_index: int = 0,
                       to_index: int = ...) -> int:
        index_location = self._last_index_location(location, from_index, to_index)
        return self._reverse_index(index_location)


def _start_and_end(x: Union['data.Label', float], end: Optional[float] = None) -> 'data.Location':
    if end is None:
        x = _location(x)
        start = x.start_index
        end = x.end_index
    else:
        if not isinstance(x, int) and not isinstance(x, float):
            raise TypeError
        if not isinstance(end, int) and not isinstance(end, float):
            raise TypeError
        start = x
        if end is None:
            end = float('inf')
    return _labels.Location(start, end)


def _location(x):
    if isinstance(x, _labels.Label):
        x = x.location
    if not isinstance(x, _labels.Location):
        raise TypeError
    return x


class _LabelIndex(LabelIndex[L]):
    __slots__ = ('_distinct', '_view', '_ascending', '_adapter', '__reversed')

    def __init__(self, distinct: bool,
                 view: _View,
                 ascending: bool = True,
                 reversed_index: '_LabelIndex[L]' = None,
                 adapter: Optional['data.ProtoLabelAdapter'] = None):
        self._distinct = distinct
        self._view = view
        self._ascending = ascending
        self._adapter = adapter
        if reversed_index is not None:
            self.__reversed = reversed_index

    @property
    def distinct(self) -> bool:
        return self._distinct

    @property
    def adapter(self) -> 'data.ProtoLabelAdapter[L]':
        return self._adapter

    @adapter.setter
    def adapter(self, value: 'data.ProtoLabelAdapter[L]'):
        self._adapter = value

    def filter(self, fn: Callable[['data.Label'], bool]) -> 'data.LabelIndex[L]':
        return self._filtered_or_empty(fn)

    @property
    def _reversed(self) -> '_LabelIndex[L]':
        try:
            reversed_label_index = self.__reversed
        except AttributeError:
            reversed_label_index = _LabelIndex(distinct=self.distinct,
                                               view=self._view.reversed(),
                                               ascending=False,
                                               reversed_index=self)
            self.__reversed = reversed_label_index
        return reversed_label_index

    def __getitem__(self, idx: Union[int, slice]) -> Union[L, 'data.LabelIndex[L]']:
        if isinstance(idx, int):
            return self._view[idx]
        elif isinstance(idx, slice):
            return _LabelIndex(self.distinct, self._view[idx], adapter=self.adapter)
        else:
            raise TypeError("Index must be int or slice.")

    def at(self, x, end=None):
        start, end = _start_and_end(x, end)
        return self._bounded_or_empty(min_start=start, max_start=start, min_end=end, max_end=end)

    def __contains__(self, item: Any) -> bool:
        try:
            self.index(item)
            return True
        except ValueError:
            return False

    def __len__(self) -> int:
        return len(self._view)

    def __iter__(self) -> Iterator[L]:
        return iter(self._view)

    def __reversed__(self) -> Iterator[L]:
        return iter(self._reversed)

    def index(self, x: Any, start: int = ..., end: int = ...) -> int:
        if not isinstance(x, _labels.Label):
            raise ValueError
        for i in range(self._view.index_location(x.location), len(self._view)):
            if self._view[i] == x:
                return i
        raise ValueError

    def count(self, x: Any) -> int:
        if not isinstance(x, _labels.Label):
            return False
        try:
            i = self._view.index_location(x.location)
        except ValueError:
            return 0
        count = 0
        for label in self._view[i:]:
            if not label.location == x.location:
                break
            if label == x:
                count += 1
        return count

    def covering(self,
                 x: Union['data.Label', float],
                 end: Optional[float] = None) -> 'data.LabelIndex[L]':
        start, end = _start_and_end(x, end)
        return self._bounded_or_empty(max_start=start, min_end=end)

    def overlapping(self,
                    x: Union['data.Label', float],
                    end: Optional[float] = None) -> 'data.LabelIndex[L]':
        start, end = _start_and_end(x, end)
        return self._bounded_or_empty(max_start=end - 1, min_end=start + 1)

    def inside(self,
               x: Union['data.Label', float],
               end: Optional[float] = None) -> 'data.LabelIndex[L]':
        start, end = _start_and_end(x, end)
        return self._bounded_or_empty(start, end - 1, start, end)

    def beginning_inside(self,
                         x: Union['data.Label', float],
                         end: Optional[float] = None) -> 'data.LabelIndex[L]':
        start, end = _start_and_end(x, end)
        return self._bounded_or_empty(min_start=start, max_start=end - 1)

    def ascending(self) -> 'data.LabelIndex[L]':
        if self._ascending:
            return self
        else:
            return self._reversed

    def descending(self) -> 'data.LabelIndex[L]':
        if not self._ascending:
            return self
        else:
            return self._reversed

    def __repr__(self):
        return ("label_index(["
                + ", ".join(repr(l) for l in self)
                + "], distinct={})".format(self.distinct))

    def __eq__(self, other):
        if len(self) != len(other):
            return False
        for s, o in zip(self, other):
            if s != o:
                return False
        return True

    def _bounded_or_empty(self,
                          min_start: float = 0,
                          max_start: float = float('inf'),
                          min_end: float = 0,
                          max_end: float = float('inf')):
        try:
            bounded_view = self._view.bound(min_start=min_start, max_start=max_start,
                                            min_end=min_end, max_end=max_end)
        except ValueError:
            return _Empty(self.distinct, self.adapter)
        return _LabelIndex(self.distinct, bounded_view, adapter=self.adapter)

    def _filtered_or_empty(self, fn: Callable[['data.Label'], bool]):
        try:
            filtered_view = self._view.filter(fn)
        except ValueError:
            return _Empty(self.distinct, self.adapter)
        return _LabelIndex(self.distinct, filtered_view, adapter=self.adapter)


class _Empty(LabelIndex):
    __slots__ = ('_distinct', '_adapter')

    @property
    def adapter(self) -> Optional['data.ProtoLabelAdapter']:
        return self._adapter

    def __init__(self, distinct, adapter=None):
        self._distinct = distinct
        self._adapter = adapter

    @property
    def distinct(self) -> bool:
        return self._distinct

    def __getitem__(self, idx):
        raise IndexError

    def filter(self, fn: Callable[['data.Label'], bool]) -> 'data.LabelIndex[L]':
        return self

    def at(self,
           x: Union['data.Label', 'data.Location', float],
           end: Optional[float] = None) -> 'data.LabelIndex[L]':
        return self

    def __len__(self) -> int:
        return 0

    def __contains__(self, item: Any) -> bool:
        return False

    def __iter__(self) -> Iterator[L]:
        return iter([])

    def __reversed__(self) -> 'data.LabelIndex[L]':
        return self

    def index(self, x: Any, start: int = ..., end: int = ...) -> int:
        raise ValueError

    def count(self, x: Any) -> int:
        return 0

    def covering(self,
                 x: Union['data.Label', float],
                 end: Optional[float] = None) -> 'data.LabelIndex[L]':
        return self

    def inside(self,
               x: Union['data.Label', float],
               end: Optional[float] = None) -> 'data.LabelIndex[L]':
        return self

    def overlapping(self,
                    x: Union['data.Label', float],
                    end: Optional[float] = None) -> 'data.LabelIndex[L]':
        return self

    def beginning_inside(self,
                         x: Union['data.Label', float],
                         end: Optional[float] = None) -> 'data.LabelIndex[L]':
        return self

    def ascending(self) -> 'data.LabelIndex[L]':
        return self

    def descending(self) -> 'data.LabelIndex[L]':
        return self

    def __eq__(self, other):
        if len(self) != len(other):
            return False
        for s, o in zip(self, other):
            if s != o:
                return False
        return True

    def __repr__(self):
        return "label_index([], distinct={})".format(self.distinct)



# Copyright 2019 Regents of the University of Minnesota.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from mtap import Location, Document, GenericLabel


def test_location_equals():
    assert Location(1, 3) == Location(1, 3)


def test_text_from_document():
    d = Document('plaintext', text='This is text.')
    assert GenericLabel(5, 7, document=d).text == 'is'


def test_label_location():
    l = GenericLabel(0, 10)
    assert l.location == Location(0, 10)


def test_location_relative_to():
    first = Location(10, 20)
    assert first.relative_to(5) == Location(5, 15)
    assert first.relative_to(GenericLabel(5, 25)) == Location(5, 15)
    assert first.relative_to(Location(5, 25)) == Location(5, 15)


def test_location_offset_by():
    first = Location(0, 5)
    assert first.offset_by(10) == Location(10, 15)
    assert first.offset_by(Location(10, 25)) == Location(10, 15)
    assert first.offset_by(GenericLabel(10, 25)) == Location(10, 15)



# Copyright 2019 Regents of the University of Minnesota.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from io import StringIO

from mtap import Event
from mtap.metrics import Accuracy, Metrics, FirstTokenConfusion, ConfusionMatrix


def test_accuracy():
    with Event(event_id='1') as event:
        doc = event.create_document('test', 'This is some text.')
        with doc.get_labeler('tested') as tested:
            tested(0, 5)
            tested(6, 10)
            tested(11, 20)
            tested(21, 29)
            tested(31, 39)

        with doc.get_labeler('target') as target:
            target(0, 5)
            target(6, 10)
            target(11, 20)
            target(21, 30)
            target(31, 39)

        acc = Accuracy()
        metrics = Metrics(acc, tested='tested', target='target')
        metrics.process_document(doc, params={})
        assert abs(acc.value - 0.8) < 1e-6


def test_any():
    with Event(event_id='1') as event:
        doc = event.create_document('test', 'This is some text.')
        with doc.get_labeler('tested') as tested:
            tested(0, 5, x=1)
            tested(0, 5, x=3)
        with doc.get_labeler('target') as target:
            target(0, 5, x=1)
            target(6, 10, x=2)

        acc = Accuracy(mode='any')
        metrics = Metrics(acc, tested='tested', target='target')
        metrics.process_document(doc, params={})
        assert abs(acc.value - 0.5) < 1e-6


def test_fields():
    with Event(event_id='1') as event:
        doc = event.create_document('test', 'This is some text.')
        with doc.get_labeler('tested') as tested:
            tested(0, 5, x=1, y=3)
            tested(6, 10, x=3, y=4)
        with doc.get_labeler('target') as target:
            target(0, 5, x=1, y=5)
            target(6, 10, x=2, y=6)

        acc = Accuracy(fields=['x'])
        metrics = Metrics(acc, tested='tested', target='target')
        metrics.process_document(doc, params={})
        assert abs(acc.value - 0.5) < 1e-6


def test_boundary_fuzz_equals():
    with Event(event_id='1') as event:
        doc = event.create_document('test', 'This is some text.')
        with doc.get_labeler('tested') as tested:
            tested(0, 5)
            tested(6, 10)
            tested(11, 20)
            tested(21, 29)
            tested(31, 39)

        with doc.get_labeler('target') as target:
            target(0, 6)
            target(7, 10)
            target(11, 19)
            target(20, 30)
            target(49, 50)

        acc = Accuracy(boundary_fuzz=1)
        metrics = Metrics(acc, tested='tested', target='target')
        metrics.process_document(doc, params={})
        assert abs(acc.value - 0.8) < 1e-6


def test_begin_token_precision_recall_f1():
    with Event() as event:
        doc = event.create_document('test', 'The quick brown fox jumps over the lazy dog.')
        with doc.get_labeler('tested') as label_tested:
            label_tested(0, 9)
            label_tested(10, 19)
            label_tested(20, 44)
        with doc.get_labeler('target') as label_target:
            label_target(0, 19)
            label_target(20, 30)
            label_target(31, 44)

        metric = FirstTokenConfusion()
        metric.update(doc, doc.labels['tested'], doc.labels['target'])
        assert metric.precision == 2 / 3
        assert metric.recall == 2 / 3
        assert metric.f1 == 2 / 3


def test_bin_classification_iadd():
    first = ConfusionMatrix(
        true_positives=2,
        false_positives=1,
        false_negatives=1
    )
    second = ConfusionMatrix(
        true_positives=3,
        false_positives=2,
        false_negatives=5
    )
    first += second
    assert first.true_positives == 5
    assert first.false_positives == 3
    assert first.false_negatives == 6


def test_print_debug_fn():
    event = Event()
    doc = event.create_document('test', 'The quick brown fox jumps over the lazy dog.')
    with doc.get_labeler('target') as label_target:
        label_target(16, 19)
    with doc.get_labeler('tested') as label_tested:
        label_tested(10, 15)

    string_io = StringIO()
    metric = FirstTokenConfusion(print_debug='fn', debug_handle=string_io)
    metric.update(doc, doc.labels['tested'], doc.labels['target'])
    assert string_io.getvalue() == 'False Negatives\nThe quick brown {fox} jumps over the lazy dog.\n\n'


def test_print_debug_fp():
    event = Event()
    doc = event.create_document('test', 'The quick brown fox jumps over the lazy dog.')
    with doc.get_labeler('target') as label_target:
        label_target(16, 19)
    with doc.get_labeler('tested') as label_tested:
        label_tested(10, 15)

    string_io = StringIO()
    metric = FirstTokenConfusion(print_debug='fp', debug_handle=string_io)
    metric.update(doc, doc.labels['tested'], doc.labels['target'])
    assert string_io.getvalue() == 'False Positives\nThe quick {brown} fox jumps over the lazy dog.\n\n'


def test_print_debug_all():
    event = Event()
    doc = event.create_document('test', 'The quick brown fox jumps over the lazy dog.')
    with doc.get_labeler('target') as label_target:
        label_target(16, 19)
    with doc.get_labeler('tested') as label_tested:
        label_tested(10, 15)

    string_io = StringIO()
    metric = FirstTokenConfusion(print_debug='all', debug_handle=string_io)
    metric.update(doc, doc.labels['tested'], doc.labels['target'])
    assert string_io.getvalue() == 'False Positives\nThe quick {brown} fox jumps over the lazy dog.\n\nFalse Negatives\nThe quick brown {fox} jumps over the lazy dog.\n\n'



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from mtap.api.v1 import events_pb2 as mtap_dot_api_dot_v1_dot_events__pb2


class EventsStub(object):
    """The events service, which stores Events (the top level discrete processing
    objects a mapping of strings to Documents and strings to strings metadata),
    Documents (a text string and a mapping of strings to LabelIndices), and
    LabelIndices (collections of objects which mark locations in text having
    some meaning).
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetEventsInstanceId = channel.unary_unary(
                '/mtap.api.v1.Events/GetEventsInstanceId',
                request_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetEventsInstanceIdRequest.SerializeToString,
                response_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetEventsInstanceIdResponse.FromString,
                )
        self.OpenEvent = channel.unary_unary(
                '/mtap.api.v1.Events/OpenEvent',
                request_serializer=mtap_dot_api_dot_v1_dot_events__pb2.OpenEventRequest.SerializeToString,
                response_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.OpenEventResponse.FromString,
                )
        self.CloseEvent = channel.unary_unary(
                '/mtap.api.v1.Events/CloseEvent',
                request_serializer=mtap_dot_api_dot_v1_dot_events__pb2.CloseEventRequest.SerializeToString,
                response_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.CloseEventResponse.FromString,
                )
        self.GetAllMetadata = channel.unary_unary(
                '/mtap.api.v1.Events/GetAllMetadata',
                request_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetAllMetadataRequest.SerializeToString,
                response_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetAllMetadataResponse.FromString,
                )
        self.AddMetadata = channel.unary_unary(
                '/mtap.api.v1.Events/AddMetadata',
                request_serializer=mtap_dot_api_dot_v1_dot_events__pb2.AddMetadataRequest.SerializeToString,
                response_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.AddMetadataResponse.FromString,
                )
        self.AddDocument = channel.unary_unary(
                '/mtap.api.v1.Events/AddDocument',
                request_serializer=mtap_dot_api_dot_v1_dot_events__pb2.AddDocumentRequest.SerializeToString,
                response_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.AddDocumentResponse.FromString,
                )
        self.GetAllDocumentNames = channel.unary_unary(
                '/mtap.api.v1.Events/GetAllDocumentNames',
                request_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetAllDocumentNamesRequest.SerializeToString,
                response_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetAllDocumentNamesResponse.FromString,
                )
        self.GetDocumentText = channel.unary_unary(
                '/mtap.api.v1.Events/GetDocumentText',
                request_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetDocumentTextRequest.SerializeToString,
                response_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetDocumentTextResponse.FromString,
                )
        self.GetLabelIndicesInfo = channel.unary_unary(
                '/mtap.api.v1.Events/GetLabelIndicesInfo',
                request_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetLabelIndicesInfoRequest.SerializeToString,
                response_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetLabelIndicesInfoResponse.FromString,
                )
        self.AddLabels = channel.unary_unary(
                '/mtap.api.v1.Events/AddLabels',
                request_serializer=mtap_dot_api_dot_v1_dot_events__pb2.AddLabelsRequest.SerializeToString,
                response_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.AddLabelsResponse.FromString,
                )
        self.GetLabels = channel.unary_unary(
                '/mtap.api.v1.Events/GetLabels',
                request_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetLabelsRequest.SerializeToString,
                response_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetLabelsResponse.FromString,
                )
        self.GetAllBinaryDataNames = channel.unary_unary(
                '/mtap.api.v1.Events/GetAllBinaryDataNames',
                request_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetAllBinaryDataNamesRequest.SerializeToString,
                response_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetAllBinaryDataNamesResponse.FromString,
                )
        self.AddBinaryData = channel.unary_unary(
                '/mtap.api.v1.Events/AddBinaryData',
                request_serializer=mtap_dot_api_dot_v1_dot_events__pb2.AddBinaryDataRequest.SerializeToString,
                response_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.AddBinaryDataResponse.FromString,
                )
        self.GetBinaryData = channel.unary_unary(
                '/mtap.api.v1.Events/GetBinaryData',
                request_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetBinaryDataRequest.SerializeToString,
                response_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetBinaryDataResponse.FromString,
                )


class EventsServicer(object):
    """The events service, which stores Events (the top level discrete processing
    objects a mapping of strings to Documents and strings to strings metadata),
    Documents (a text string and a mapping of strings to LabelIndices), and
    LabelIndices (collections of objects which mark locations in text having
    some meaning).
    """

    def GetEventsInstanceId(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def OpenEvent(self, request, context):
        """Acquires a lease for an event, which will prevent this service
        from deleting the event while the lease is still valid. Can be used to
        either create new events or return existing events.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CloseEvent(self, request, context):
        """Cedes a lease for an event, allowing this service to delete the
        event if no active leases remain.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAllMetadata(self, request, context):
        """Endpoint to get all metadata associated with an event.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddMetadata(self, request, context):
        """Adds a new metadata entry to an event.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddDocument(self, request, context):
        """Adds a new document to an event.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAllDocumentNames(self, request, context):
        """Returns the names keys of all documents that are stored on an event.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDocumentText(self, request, context):
        """Returns the text of a document.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetLabelIndicesInfo(self, request, context):
        """Returns the names keys of all label indices that are stored on a document.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddLabels(self, request, context):
        """Adds a single label index to the document.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetLabels(self, request, context):
        """Gets all of the labels for a single label index.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAllBinaryDataNames(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddBinaryData(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBinaryData(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_EventsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetEventsInstanceId': grpc.unary_unary_rpc_method_handler(
                    servicer.GetEventsInstanceId,
                    request_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetEventsInstanceIdRequest.FromString,
                    response_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetEventsInstanceIdResponse.SerializeToString,
            ),
            'OpenEvent': grpc.unary_unary_rpc_method_handler(
                    servicer.OpenEvent,
                    request_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.OpenEventRequest.FromString,
                    response_serializer=mtap_dot_api_dot_v1_dot_events__pb2.OpenEventResponse.SerializeToString,
            ),
            'CloseEvent': grpc.unary_unary_rpc_method_handler(
                    servicer.CloseEvent,
                    request_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.CloseEventRequest.FromString,
                    response_serializer=mtap_dot_api_dot_v1_dot_events__pb2.CloseEventResponse.SerializeToString,
            ),
            'GetAllMetadata': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllMetadata,
                    request_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetAllMetadataRequest.FromString,
                    response_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetAllMetadataResponse.SerializeToString,
            ),
            'AddMetadata': grpc.unary_unary_rpc_method_handler(
                    servicer.AddMetadata,
                    request_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.AddMetadataRequest.FromString,
                    response_serializer=mtap_dot_api_dot_v1_dot_events__pb2.AddMetadataResponse.SerializeToString,
            ),
            'AddDocument': grpc.unary_unary_rpc_method_handler(
                    servicer.AddDocument,
                    request_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.AddDocumentRequest.FromString,
                    response_serializer=mtap_dot_api_dot_v1_dot_events__pb2.AddDocumentResponse.SerializeToString,
            ),
            'GetAllDocumentNames': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllDocumentNames,
                    request_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetAllDocumentNamesRequest.FromString,
                    response_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetAllDocumentNamesResponse.SerializeToString,
            ),
            'GetDocumentText': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDocumentText,
                    request_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetDocumentTextRequest.FromString,
                    response_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetDocumentTextResponse.SerializeToString,
            ),
            'GetLabelIndicesInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetLabelIndicesInfo,
                    request_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetLabelIndicesInfoRequest.FromString,
                    response_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetLabelIndicesInfoResponse.SerializeToString,
            ),
            'AddLabels': grpc.unary_unary_rpc_method_handler(
                    servicer.AddLabels,
                    request_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.AddLabelsRequest.FromString,
                    response_serializer=mtap_dot_api_dot_v1_dot_events__pb2.AddLabelsResponse.SerializeToString,
            ),
            'GetLabels': grpc.unary_unary_rpc_method_handler(
                    servicer.GetLabels,
                    request_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetLabelsRequest.FromString,
                    response_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetLabelsResponse.SerializeToString,
            ),
            'GetAllBinaryDataNames': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllBinaryDataNames,
                    request_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetAllBinaryDataNamesRequest.FromString,
                    response_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetAllBinaryDataNamesResponse.SerializeToString,
            ),
            'AddBinaryData': grpc.unary_unary_rpc_method_handler(
                    servicer.AddBinaryData,
                    request_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.AddBinaryDataRequest.FromString,
                    response_serializer=mtap_dot_api_dot_v1_dot_events__pb2.AddBinaryDataResponse.SerializeToString,
            ),
            'GetBinaryData': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBinaryData,
                    request_deserializer=mtap_dot_api_dot_v1_dot_events__pb2.GetBinaryDataRequest.FromString,
                    response_serializer=mtap_dot_api_dot_v1_dot_events__pb2.GetBinaryDataResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'mtap.api.v1.Events', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Events(object):
    """The events service, which stores Events (the top level discrete processing
    objects a mapping of strings to Documents and strings to strings metadata),
    Documents (a text string and a mapping of strings to LabelIndices), and
    LabelIndices (collections of objects which mark locations in text having
    some meaning).
    """

    @staticmethod
    def GetEventsInstanceId(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/mtap.api.v1.Events/GetEventsInstanceId',
            mtap_dot_api_dot_v1_dot_events__pb2.GetEventsInstanceIdRequest.SerializeToString,
            mtap_dot_api_dot_v1_dot_events__pb2.GetEventsInstanceIdResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def OpenEvent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/mtap.api.v1.Events/OpenEvent',
            mtap_dot_api_dot_v1_dot_events__pb2.OpenEventRequest.SerializeToString,
            mtap_dot_api_dot_v1_dot_events__pb2.OpenEventResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CloseEvent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/mtap.api.v1.Events/CloseEvent',
            mtap_dot_api_dot_v1_dot_events__pb2.CloseEventRequest.SerializeToString,
            mtap_dot_api_dot_v1_dot_events__pb2.CloseEventResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAllMetadata(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/mtap.api.v1.Events/GetAllMetadata',
            mtap_dot_api_dot_v1_dot_events__pb2.GetAllMetadataRequest.SerializeToString,
            mtap_dot_api_dot_v1_dot_events__pb2.GetAllMetadataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddMetadata(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/mtap.api.v1.Events/AddMetadata',
            mtap_dot_api_dot_v1_dot_events__pb2.AddMetadataRequest.SerializeToString,
            mtap_dot_api_dot_v1_dot_events__pb2.AddMetadataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddDocument(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/mtap.api.v1.Events/AddDocument',
            mtap_dot_api_dot_v1_dot_events__pb2.AddDocumentRequest.SerializeToString,
            mtap_dot_api_dot_v1_dot_events__pb2.AddDocumentResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAllDocumentNames(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/mtap.api.v1.Events/GetAllDocumentNames',
            mtap_dot_api_dot_v1_dot_events__pb2.GetAllDocumentNamesRequest.SerializeToString,
            mtap_dot_api_dot_v1_dot_events__pb2.GetAllDocumentNamesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDocumentText(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/mtap.api.v1.Events/GetDocumentText',
            mtap_dot_api_dot_v1_dot_events__pb2.GetDocumentTextRequest.SerializeToString,
            mtap_dot_api_dot_v1_dot_events__pb2.GetDocumentTextResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetLabelIndicesInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/mtap.api.v1.Events/GetLabelIndicesInfo',
            mtap_dot_api_dot_v1_dot_events__pb2.GetLabelIndicesInfoRequest.SerializeToString,
            mtap_dot_api_dot_v1_dot_events__pb2.GetLabelIndicesInfoResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddLabels(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/mtap.api.v1.Events/AddLabels',
            mtap_dot_api_dot_v1_dot_events__pb2.AddLabelsRequest.SerializeToString,
            mtap_dot_api_dot_v1_dot_events__pb2.AddLabelsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetLabels(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/mtap.api.v1.Events/GetLabels',
            mtap_dot_api_dot_v1_dot_events__pb2.GetLabelsRequest.SerializeToString,
            mtap_dot_api_dot_v1_dot_events__pb2.GetLabelsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAllBinaryDataNames(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/mtap.api.v1.Events/GetAllBinaryDataNames',
            mtap_dot_api_dot_v1_dot_events__pb2.GetAllBinaryDataNamesRequest.SerializeToString,
            mtap_dot_api_dot_v1_dot_events__pb2.GetAllBinaryDataNamesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddBinaryData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/mtap.api.v1.Events/AddBinaryData',
            mtap_dot_api_dot_v1_dot_events__pb2.AddBinaryDataRequest.SerializeToString,
            mtap_dot_api_dot_v1_dot_events__pb2.AddBinaryDataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetBinaryData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/mtap.api.v1.Events/GetBinaryData',
            mtap_dot_api_dot_v1_dot_events__pb2.GetBinaryDataRequest.SerializeToString,
            mtap_dot_api_dot_v1_dot_events__pb2.GetBinaryDataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mtap/api/v1/events.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18mtap/api/v1/events.proto\x12\x0bmtap.api.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x19google/protobuf/any.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1cgoogle/protobuf/struct.proto\"\x1c\n\x1aGetEventsInstanceIdRequest\"2\n\x1bGetEventsInstanceIdResponse\x12\x13\n\x0binstance_id\x18\x01 \x01(\t\"p\n\x10OpenEventRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x17\n\x0fonly_create_new\x18\x02 \x01(\x08\x12\x31\n\x0elease_duration\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\"6\n\x11OpenEventResponse\x12\x0f\n\x07\x63reated\x18\x01 \x01(\x08\x12\x10\n\x08lease_id\x18\x02 \x01(\x05\"7\n\x11\x43loseEventRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x10\n\x08lease_id\x18\x02 \x01(\x05\"%\n\x12\x43loseEventResponse\x12\x0f\n\x07\x64\x65leted\x18\x01 \x01(\x08\")\n\x15GetAllMetadataRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\"\x8e\x01\n\x16GetAllMetadataResponse\x12\x43\n\x08metadata\x18\x01 \x03(\x0b\x32\x31.mtap.api.v1.GetAllMetadataResponse.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"B\n\x12\x41\x64\x64MetadataRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\"\x15\n\x13\x41\x64\x64MetadataResponse\"0\n\x1cGetAllBinaryDataNamesRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\":\n\x1dGetAllBinaryDataNamesResponse\x12\x19\n\x11\x62inary_data_names\x18\x01 \x03(\t\"W\n\x14\x41\x64\x64\x42inaryDataRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x18\n\x10\x62inary_data_name\x18\x02 \x01(\t\x12\x13\n\x0b\x62inary_data\x18\x03 \x01(\x0c\"\x17\n\x15\x41\x64\x64\x42inaryDataResponse\"B\n\x14GetBinaryDataRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x18\n\x10\x62inary_data_name\x18\x02 \x01(\t\",\n\x15GetBinaryDataResponse\x12\x13\n\x0b\x62inary_data\x18\x01 \x01(\x0c\"K\n\x12\x41\x64\x64\x44ocumentRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x15\n\rdocument_name\x18\x02 \x01(\t\x12\x0c\n\x04text\x18\x03 \x01(\t\"\x15\n\x13\x41\x64\x64\x44ocumentResponse\".\n\x1aGetAllDocumentNamesRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\"5\n\x1bGetAllDocumentNamesResponse\x12\x16\n\x0e\x64ocument_names\x18\x01 \x03(\t\"A\n\x16GetDocumentTextRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x15\n\rdocument_name\x18\x02 \x01(\t\"\'\n\x17GetDocumentTextResponse\x12\x0c\n\x04text\x18\x01 \x01(\t\"E\n\x1aGetLabelIndicesInfoRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x15\n\rdocument_name\x18\x02 \x01(\t\"\xa6\x02\n\x1bGetLabelIndicesInfoResponse\x12R\n\x11label_index_infos\x18\x01 \x03(\x0b\x32\x37.mtap.api.v1.GetLabelIndicesInfoResponse.LabelIndexInfo\x1a\xb2\x01\n\x0eLabelIndexInfo\x12\x12\n\nindex_name\x18\x01 \x01(\t\x12T\n\x04type\x18\x02 \x01(\x0e\x32\x46.mtap.api.v1.GetLabelIndicesInfoResponse.LabelIndexInfo.LabelIndexType\"6\n\x0eLabelIndexType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07GENERIC\x10\x01\x12\n\n\x06\x43USTOM\x10\x02\"\xd9\x01\n\x10\x41\x64\x64LabelsRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x15\n\rdocument_name\x18\x02 \x01(\t\x12\x12\n\nindex_name\x18\x03 \x01(\t\x12\x34\n\x0egeneric_labels\x18\x04 \x01(\x0b\x32\x1a.mtap.api.v1.GenericLabelsH\x00\x12-\n\rcustom_labels\x18\x05 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00\x12\x19\n\x11no_key_validation\x18\x06 \x01(\x08\x42\x08\n\x06labels\"\x13\n\x11\x41\x64\x64LabelsResponse\"O\n\x10GetLabelsRequest\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x15\n\rdocument_name\x18\x02 \x01(\t\x12\x12\n\nindex_name\x18\x03 \x01(\t\"O\n\rGenericLabels\x12\x13\n\x0bis_distinct\x18\x01 \x01(\x08\x12)\n\x06labels\x18\x02 \x03(\x0b\x32\x19.mtap.api.v1.GenericLabel\"\xa3\x01\n\x0cGenericLabel\x12\x12\n\nidentifier\x18\x01 \x01(\r\x12\x13\n\x0bstart_index\x18\x02 \x01(\r\x12\x11\n\tend_index\x18\x03 \x01(\r\x12\'\n\x06\x66ields\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\x12.\n\rreference_ids\x18\x05 \x01(\x0b\x32\x17.google.protobuf.Struct\"\x82\x01\n\x11GetLabelsResponse\x12\x34\n\x0egeneric_labels\x18\x01 \x01(\x0b\x32\x1a.mtap.api.v1.GenericLabelsH\x00\x12-\n\rcustom_labels\x18\x02 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00\x42\x08\n\x06labels2\xd8\x0f\n\x06\x45vents\x12\x88\x01\n\x13GetEventsInstanceId\x12\'.mtap.api.v1.GetEventsInstanceIdRequest\x1a(.mtap.api.v1.GetEventsInstanceIdResponse\"\x1e\x82\xd3\xe4\x93\x02\x18\x12\x16/v1/events/instance_id\x12l\n\tOpenEvent\x12\x1d.mtap.api.v1.OpenEventRequest\x1a\x1e.mtap.api.v1.OpenEventResponse\" \x82\xd3\xe4\x93\x02\x1a\"\x15/v1/events/{event_id}:\x01*\x12l\n\nCloseEvent\x12\x1e.mtap.api.v1.CloseEventRequest\x1a\x1f.mtap.api.v1.CloseEventResponse\"\x1d\x82\xd3\xe4\x93\x02\x17*\x15/v1/events/{event_id}\x12\x81\x01\n\x0eGetAllMetadata\x12\".mtap.api.v1.GetAllMetadataRequest\x1a#.mtap.api.v1.GetAllMetadataResponse\"&\x82\xd3\xe4\x93\x02 \x12\x1e/v1/events/{event_id}/metadata\x12\x81\x01\n\x0b\x41\x64\x64Metadata\x12\x1f.mtap.api.v1.AddMetadataRequest\x1a .mtap.api.v1.AddMetadataResponse\"/\x82\xd3\xe4\x93\x02)\"$/v1/events/{event_id}/metadata/{key}:\x01*\x12\x8c\x01\n\x0b\x41\x64\x64\x44ocument\x12\x1f.mtap.api.v1.AddDocumentRequest\x1a .mtap.api.v1.AddDocumentResponse\":\x82\xd3\xe4\x93\x02\x34\"//v1/events/{event_id}/documents/{document_name}:\x01*\x12\x91\x01\n\x13GetAllDocumentNames\x12\'.mtap.api.v1.GetAllDocumentNamesRequest\x1a(.mtap.api.v1.GetAllDocumentNamesResponse\"\'\x82\xd3\xe4\x93\x02!\x12\x1f/v1/events/{event_id}/documents\x12\x95\x01\n\x0fGetDocumentText\x12#.mtap.api.v1.GetDocumentTextRequest\x1a$.mtap.api.v1.GetDocumentTextResponse\"7\x82\xd3\xe4\x93\x02\x31\x12//v1/events/{event_id}/documents/{document_name}\x12\xa8\x01\n\x13GetLabelIndicesInfo\x12\'.mtap.api.v1.GetLabelIndicesInfoRequest\x1a(.mtap.api.v1.GetLabelIndicesInfoResponse\">\x82\xd3\xe4\x93\x02\x38\x12\x36/v1/events/{event_id}/documents/{document_name}/labels\x12\x9a\x01\n\tAddLabels\x12\x1d.mtap.api.v1.AddLabelsRequest\x1a\x1e.mtap.api.v1.AddLabelsResponse\"N\x82\xd3\xe4\x93\x02H\"C/v1/events/{event_id}/documents/{document_name}/labels/{index_name}:\x01*\x12\x97\x01\n\tGetLabels\x12\x1d.mtap.api.v1.GetLabelsRequest\x1a\x1e.mtap.api.v1.GetLabelsResponse\"K\x82\xd3\xe4\x93\x02\x45\x12\x43/v1/events/{event_id}/documents/{document_name}/labels/{index_name}\x12\x96\x01\n\x15GetAllBinaryDataNames\x12).mtap.api.v1.GetAllBinaryDataNamesRequest\x1a*.mtap.api.v1.GetAllBinaryDataNamesResponse\"&\x82\xd3\xe4\x93\x02 \x12\x1e/v1/events/{event_id}/binaries\x12\x94\x01\n\rAddBinaryData\x12!.mtap.api.v1.AddBinaryDataRequest\x1a\".mtap.api.v1.AddBinaryDataResponse\"<\x82\xd3\xe4\x93\x02\x36\"1/v1/events/{event_id}/binaries/{binary_data_name}:\x01*\x12\x91\x01\n\rGetBinaryData\x12!.mtap.api.v1.GetBinaryDataRequest\x1a\".mtap.api.v1.GetBinaryDataResponse\"9\x82\xd3\xe4\x93\x02\x33\x12\x31/v1/events/{event_id}/binaries/{binary_data_name}B\x1b\n\x19\x65\x64u.umn.nlpie.mtap.api.v1b\x06proto3')



_GETEVENTSINSTANCEIDREQUEST = DESCRIPTOR.message_types_by_name['GetEventsInstanceIdRequest']
_GETEVENTSINSTANCEIDRESPONSE = DESCRIPTOR.message_types_by_name['GetEventsInstanceIdResponse']
_OPENEVENTREQUEST = DESCRIPTOR.message_types_by_name['OpenEventRequest']
_OPENEVENTRESPONSE = DESCRIPTOR.message_types_by_name['OpenEventResponse']
_CLOSEEVENTREQUEST = DESCRIPTOR.message_types_by_name['CloseEventRequest']
_CLOSEEVENTRESPONSE = DESCRIPTOR.message_types_by_name['CloseEventResponse']
_GETALLMETADATAREQUEST = DESCRIPTOR.message_types_by_name['GetAllMetadataRequest']
_GETALLMETADATARESPONSE = DESCRIPTOR.message_types_by_name['GetAllMetadataResponse']
_GETALLMETADATARESPONSE_METADATAENTRY = _GETALLMETADATARESPONSE.nested_types_by_name['MetadataEntry']
_ADDMETADATAREQUEST = DESCRIPTOR.message_types_by_name['AddMetadataRequest']
_ADDMETADATARESPONSE = DESCRIPTOR.message_types_by_name['AddMetadataResponse']
_GETALLBINARYDATANAMESREQUEST = DESCRIPTOR.message_types_by_name['GetAllBinaryDataNamesRequest']
_GETALLBINARYDATANAMESRESPONSE = DESCRIPTOR.message_types_by_name['GetAllBinaryDataNamesResponse']
_ADDBINARYDATAREQUEST = DESCRIPTOR.message_types_by_name['AddBinaryDataRequest']
_ADDBINARYDATARESPONSE = DESCRIPTOR.message_types_by_name['AddBinaryDataResponse']
_GETBINARYDATAREQUEST = DESCRIPTOR.message_types_by_name['GetBinaryDataRequest']
_GETBINARYDATARESPONSE = DESCRIPTOR.message_types_by_name['GetBinaryDataResponse']
_ADDDOCUMENTREQUEST = DESCRIPTOR.message_types_by_name['AddDocumentRequest']
_ADDDOCUMENTRESPONSE = DESCRIPTOR.message_types_by_name['AddDocumentResponse']
_GETALLDOCUMENTNAMESREQUEST = DESCRIPTOR.message_types_by_name['GetAllDocumentNamesRequest']
_GETALLDOCUMENTNAMESRESPONSE = DESCRIPTOR.message_types_by_name['GetAllDocumentNamesResponse']
_GETDOCUMENTTEXTREQUEST = DESCRIPTOR.message_types_by_name['GetDocumentTextRequest']
_GETDOCUMENTTEXTRESPONSE = DESCRIPTOR.message_types_by_name['GetDocumentTextResponse']
_GETLABELINDICESINFOREQUEST = DESCRIPTOR.message_types_by_name['GetLabelIndicesInfoRequest']
_GETLABELINDICESINFORESPONSE = DESCRIPTOR.message_types_by_name['GetLabelIndicesInfoResponse']
_GETLABELINDICESINFORESPONSE_LABELINDEXINFO = _GETLABELINDICESINFORESPONSE.nested_types_by_name['LabelIndexInfo']
_ADDLABELSREQUEST = DESCRIPTOR.message_types_by_name['AddLabelsRequest']
_ADDLABELSRESPONSE = DESCRIPTOR.message_types_by_name['AddLabelsResponse']
_GETLABELSREQUEST = DESCRIPTOR.message_types_by_name['GetLabelsRequest']
_GENERICLABELS = DESCRIPTOR.message_types_by_name['GenericLabels']
_GENERICLABEL = DESCRIPTOR.message_types_by_name['GenericLabel']
_GETLABELSRESPONSE = DESCRIPTOR.message_types_by_name['GetLabelsResponse']
_GETLABELINDICESINFORESPONSE_LABELINDEXINFO_LABELINDEXTYPE = _GETLABELINDICESINFORESPONSE_LABELINDEXINFO.enum_types_by_name['LabelIndexType']
GetEventsInstanceIdRequest = _reflection.GeneratedProtocolMessageType('GetEventsInstanceIdRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETEVENTSINSTANCEIDREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetEventsInstanceIdRequest)
  })
_sym_db.RegisterMessage(GetEventsInstanceIdRequest)

GetEventsInstanceIdResponse = _reflection.GeneratedProtocolMessageType('GetEventsInstanceIdResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETEVENTSINSTANCEIDRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetEventsInstanceIdResponse)
  })
_sym_db.RegisterMessage(GetEventsInstanceIdResponse)

OpenEventRequest = _reflection.GeneratedProtocolMessageType('OpenEventRequest', (_message.Message,), {
  'DESCRIPTOR' : _OPENEVENTREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.OpenEventRequest)
  })
_sym_db.RegisterMessage(OpenEventRequest)

OpenEventResponse = _reflection.GeneratedProtocolMessageType('OpenEventResponse', (_message.Message,), {
  'DESCRIPTOR' : _OPENEVENTRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.OpenEventResponse)
  })
_sym_db.RegisterMessage(OpenEventResponse)

CloseEventRequest = _reflection.GeneratedProtocolMessageType('CloseEventRequest', (_message.Message,), {
  'DESCRIPTOR' : _CLOSEEVENTREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.CloseEventRequest)
  })
_sym_db.RegisterMessage(CloseEventRequest)

CloseEventResponse = _reflection.GeneratedProtocolMessageType('CloseEventResponse', (_message.Message,), {
  'DESCRIPTOR' : _CLOSEEVENTRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.CloseEventResponse)
  })
_sym_db.RegisterMessage(CloseEventResponse)

GetAllMetadataRequest = _reflection.GeneratedProtocolMessageType('GetAllMetadataRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETALLMETADATAREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetAllMetadataRequest)
  })
_sym_db.RegisterMessage(GetAllMetadataRequest)

GetAllMetadataResponse = _reflection.GeneratedProtocolMessageType('GetAllMetadataResponse', (_message.Message,), {

  'MetadataEntry' : _reflection.GeneratedProtocolMessageType('MetadataEntry', (_message.Message,), {
    'DESCRIPTOR' : _GETALLMETADATARESPONSE_METADATAENTRY,
    '__module__' : 'mtap.api.v1.events_pb2'
    # @@protoc_insertion_point(class_scope:mtap.api.v1.GetAllMetadataResponse.MetadataEntry)
    })
  ,
  'DESCRIPTOR' : _GETALLMETADATARESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetAllMetadataResponse)
  })
_sym_db.RegisterMessage(GetAllMetadataResponse)
_sym_db.RegisterMessage(GetAllMetadataResponse.MetadataEntry)

AddMetadataRequest = _reflection.GeneratedProtocolMessageType('AddMetadataRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDMETADATAREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddMetadataRequest)
  })
_sym_db.RegisterMessage(AddMetadataRequest)

AddMetadataResponse = _reflection.GeneratedProtocolMessageType('AddMetadataResponse', (_message.Message,), {
  'DESCRIPTOR' : _ADDMETADATARESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddMetadataResponse)
  })
_sym_db.RegisterMessage(AddMetadataResponse)

GetAllBinaryDataNamesRequest = _reflection.GeneratedProtocolMessageType('GetAllBinaryDataNamesRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETALLBINARYDATANAMESREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetAllBinaryDataNamesRequest)
  })
_sym_db.RegisterMessage(GetAllBinaryDataNamesRequest)

GetAllBinaryDataNamesResponse = _reflection.GeneratedProtocolMessageType('GetAllBinaryDataNamesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETALLBINARYDATANAMESRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetAllBinaryDataNamesResponse)
  })
_sym_db.RegisterMessage(GetAllBinaryDataNamesResponse)

AddBinaryDataRequest = _reflection.GeneratedProtocolMessageType('AddBinaryDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDBINARYDATAREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddBinaryDataRequest)
  })
_sym_db.RegisterMessage(AddBinaryDataRequest)

AddBinaryDataResponse = _reflection.GeneratedProtocolMessageType('AddBinaryDataResponse', (_message.Message,), {
  'DESCRIPTOR' : _ADDBINARYDATARESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddBinaryDataResponse)
  })
_sym_db.RegisterMessage(AddBinaryDataResponse)

GetBinaryDataRequest = _reflection.GeneratedProtocolMessageType('GetBinaryDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETBINARYDATAREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetBinaryDataRequest)
  })
_sym_db.RegisterMessage(GetBinaryDataRequest)

GetBinaryDataResponse = _reflection.GeneratedProtocolMessageType('GetBinaryDataResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETBINARYDATARESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetBinaryDataResponse)
  })
_sym_db.RegisterMessage(GetBinaryDataResponse)

AddDocumentRequest = _reflection.GeneratedProtocolMessageType('AddDocumentRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDDOCUMENTREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddDocumentRequest)
  })
_sym_db.RegisterMessage(AddDocumentRequest)

AddDocumentResponse = _reflection.GeneratedProtocolMessageType('AddDocumentResponse', (_message.Message,), {
  'DESCRIPTOR' : _ADDDOCUMENTRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddDocumentResponse)
  })
_sym_db.RegisterMessage(AddDocumentResponse)

GetAllDocumentNamesRequest = _reflection.GeneratedProtocolMessageType('GetAllDocumentNamesRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETALLDOCUMENTNAMESREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetAllDocumentNamesRequest)
  })
_sym_db.RegisterMessage(GetAllDocumentNamesRequest)

GetAllDocumentNamesResponse = _reflection.GeneratedProtocolMessageType('GetAllDocumentNamesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETALLDOCUMENTNAMESRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetAllDocumentNamesResponse)
  })
_sym_db.RegisterMessage(GetAllDocumentNamesResponse)

GetDocumentTextRequest = _reflection.GeneratedProtocolMessageType('GetDocumentTextRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETDOCUMENTTEXTREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetDocumentTextRequest)
  })
_sym_db.RegisterMessage(GetDocumentTextRequest)

GetDocumentTextResponse = _reflection.GeneratedProtocolMessageType('GetDocumentTextResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETDOCUMENTTEXTRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetDocumentTextResponse)
  })
_sym_db.RegisterMessage(GetDocumentTextResponse)

GetLabelIndicesInfoRequest = _reflection.GeneratedProtocolMessageType('GetLabelIndicesInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETLABELINDICESINFOREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetLabelIndicesInfoRequest)
  })
_sym_db.RegisterMessage(GetLabelIndicesInfoRequest)

GetLabelIndicesInfoResponse = _reflection.GeneratedProtocolMessageType('GetLabelIndicesInfoResponse', (_message.Message,), {

  'LabelIndexInfo' : _reflection.GeneratedProtocolMessageType('LabelIndexInfo', (_message.Message,), {
    'DESCRIPTOR' : _GETLABELINDICESINFORESPONSE_LABELINDEXINFO,
    '__module__' : 'mtap.api.v1.events_pb2'
    # @@protoc_insertion_point(class_scope:mtap.api.v1.GetLabelIndicesInfoResponse.LabelIndexInfo)
    })
  ,
  'DESCRIPTOR' : _GETLABELINDICESINFORESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetLabelIndicesInfoResponse)
  })
_sym_db.RegisterMessage(GetLabelIndicesInfoResponse)
_sym_db.RegisterMessage(GetLabelIndicesInfoResponse.LabelIndexInfo)

AddLabelsRequest = _reflection.GeneratedProtocolMessageType('AddLabelsRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDLABELSREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddLabelsRequest)
  })
_sym_db.RegisterMessage(AddLabelsRequest)

AddLabelsResponse = _reflection.GeneratedProtocolMessageType('AddLabelsResponse', (_message.Message,), {
  'DESCRIPTOR' : _ADDLABELSRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.AddLabelsResponse)
  })
_sym_db.RegisterMessage(AddLabelsResponse)

GetLabelsRequest = _reflection.GeneratedProtocolMessageType('GetLabelsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETLABELSREQUEST,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetLabelsRequest)
  })
_sym_db.RegisterMessage(GetLabelsRequest)

GenericLabels = _reflection.GeneratedProtocolMessageType('GenericLabels', (_message.Message,), {
  'DESCRIPTOR' : _GENERICLABELS,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GenericLabels)
  })
_sym_db.RegisterMessage(GenericLabels)

GenericLabel = _reflection.GeneratedProtocolMessageType('GenericLabel', (_message.Message,), {
  'DESCRIPTOR' : _GENERICLABEL,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GenericLabel)
  })
_sym_db.RegisterMessage(GenericLabel)

GetLabelsResponse = _reflection.GeneratedProtocolMessageType('GetLabelsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETLABELSRESPONSE,
  '__module__' : 'mtap.api.v1.events_pb2'
  # @@protoc_insertion_point(class_scope:mtap.api.v1.GetLabelsResponse)
  })
_sym_db.RegisterMessage(GetLabelsResponse)

_EVENTS = DESCRIPTOR.services_by_name['Events']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031edu.umn.nlpie.mtap.api.v1'
  _GETALLMETADATARESPONSE_METADATAENTRY._options = None
  _GETALLMETADATARESPONSE_METADATAENTRY._serialized_options = b'8\001'
  _EVENTS.methods_by_name['GetEventsInstanceId']._options = None
  _EVENTS.methods_by_name['GetEventsInstanceId']._serialized_options = b'\202\323\344\223\002\030\022\026/v1/events/instance_id'
  _EVENTS.methods_by_name['OpenEvent']._options = None
  _EVENTS.methods_by_name['OpenEvent']._serialized_options = b'\202\323\344\223\002\032\"\025/v1/events/{event_id}:\001*'
  _EVENTS.methods_by_name['CloseEvent']._options = None
  _EVENTS.methods_by_name['CloseEvent']._serialized_options = b'\202\323\344\223\002\027*\025/v1/events/{event_id}'
  _EVENTS.methods_by_name['GetAllMetadata']._options = None
  _EVENTS.methods_by_name['GetAllMetadata']._serialized_options = b'\202\323\344\223\002 \022\036/v1/events/{event_id}/metadata'
  _EVENTS.methods_by_name['AddMetadata']._options = None
  _EVENTS.methods_by_name['AddMetadata']._serialized_options = b'\202\323\344\223\002)\"$/v1/events/{event_id}/metadata/{key}:\001*'
  _EVENTS.methods_by_name['AddDocument']._options = None
  _EVENTS.methods_by_name['AddDocument']._serialized_options = b'\202\323\344\223\0024\"//v1/events/{event_id}/documents/{document_name}:\001*'
  _EVENTS.methods_by_name['GetAllDocumentNames']._options = None
  _EVENTS.methods_by_name['GetAllDocumentNames']._serialized_options = b'\202\323\344\223\002!\022\037/v1/events/{event_id}/documents'
  _EVENTS.methods_by_name['GetDocumentText']._options = None
  _EVENTS.methods_by_name['GetDocumentText']._serialized_options = b'\202\323\344\223\0021\022//v1/events/{event_id}/documents/{document_name}'
  _EVENTS.methods_by_name['GetLabelIndicesInfo']._options = None
  _EVENTS.methods_by_name['GetLabelIndicesInfo']._serialized_options = b'\202\323\344\223\0028\0226/v1/events/{event_id}/documents/{document_name}/labels'
  _EVENTS.methods_by_name['AddLabels']._options = None
  _EVENTS.methods_by_name['AddLabels']._serialized_options = b'\202\323\344\223\002H\"C/v1/events/{event_id}/documents/{document_name}/labels/{index_name}:\001*'
  _EVENTS.methods_by_name['GetLabels']._options = None
  _EVENTS.methods_by_name['GetLabels']._serialized_options = b'\202\323\344\223\002E\022C/v1/events/{event_id}/documents/{document_name}/labels/{index_name}'
  _EVENTS.methods_by_name['GetAllBinaryDataNames']._options = None
  _EVENTS.methods_by_name['GetAllBinaryDataNames']._serialized_options = b'\202\323\344\223\002 \022\036/v1/events/{event_id}/binaries'
  _EVENTS.methods_by_name['AddBinaryData']._options = None
  _EVENTS.methods_by_name['AddBinaryData']._serialized_options = b'\202\323\344\223\0026\"1/v1/events/{event_id}/binaries/{binary_data_name}:\001*'
  _EVENTS.methods_by_name['GetBinaryData']._options = None
  _EVENTS.methods_by_name['GetBinaryData']._serialized_options = b'\202\323\344\223\0023\0221/v1/events/{event_id}/binaries/{binary_data_name}'
  _GETEVENTSINSTANCEIDREQUEST._serialized_start=160
  _GETEVENTSINSTANCEIDREQUEST._serialized_end=188
  _GETEVENTSINSTANCEIDRESPONSE._serialized_start=190
  _GETEVENTSINSTANCEIDRESPONSE._serialized_end=240
  _OPENEVENTREQUEST._serialized_start=242
  _OPENEVENTREQUEST._serialized_end=354
  _OPENEVENTRESPONSE._serialized_start=356
  _OPENEVENTRESPONSE._serialized_end=410
  _CLOSEEVENTREQUEST._serialized_start=412
  _CLOSEEVENTREQUEST._serialized_end=467
  _CLOSEEVENTRESPONSE._serialized_start=469
  _CLOSEEVENTRESPONSE._serialized_end=506
  _GETALLMETADATAREQUEST._serialized_start=508
  _GETALLMETADATAREQUEST._serialized_end=549
  _GETALLMETADATARESPONSE._serialized_start=552
  _GETALLMETADATARESPONSE._serialized_end=694
  _GETALLMETADATARESPONSE_METADATAENTRY._serialized_start=647
  _GETALLMETADATARESPONSE_METADATAENTRY._serialized_end=694
  _ADDMETADATAREQUEST._serialized_start=696
  _ADDMETADATAREQUEST._serialized_end=762
  _ADDMETADATARESPONSE._serialized_start=764
  _ADDMETADATARESPONSE._serialized_end=785
  _GETALLBINARYDATANAMESREQUEST._serialized_start=787
  _GETALLBINARYDATANAMESREQUEST._serialized_end=835
  _GETALLBINARYDATANAMESRESPONSE._serialized_start=837
  _GETALLBINARYDATANAMESRESPONSE._serialized_end=895
  _ADDBINARYDATAREQUEST._serialized_start=897
  _ADDBINARYDATAREQUEST._serialized_end=984
  _ADDBINARYDATARESPONSE._serialized_start=986
  _ADDBINARYDATARESPONSE._serialized_end=1009
  _GETBINARYDATAREQUEST._serialized_start=1011
  _GETBINARYDATAREQUEST._serialized_end=1077
  _GETBINARYDATARESPONSE._serialized_start=1079
  _GETBINARYDATARESPONSE._serialized_end=1123
  _ADDDOCUMENTREQUEST._serialized_start=1125
  _ADDDOCUMENTREQUEST._serialized_end=1200
  _ADDDOCUMENTRESPONSE._serialized_start=1202
  _ADDDOCUMENTRESPONSE._serialized_end=1223
  _GETALLDOCUMENTNAMESREQUEST._serialized_start=1225
  _GETALLDOCUMENTNAMESREQUEST._serialized_end=1271
  _GETALLDOCUMENTNAMESRESPONSE._serialized_start=1273
  _GETALLDOCUMENTNAMESRESPONSE._serialized_end=1326
  _GETDOCUMENTTEXTREQUEST._serialized_start=1328
  _GETDOCUMENTTEXTREQUEST._serialized_end=1393
  _GETDOCUMENTTEXTRESPONSE._serialized_start=1395
  _GETDOCUMENTTEXTRESPONSE._serialized_end=1434
  _GETLABELINDICESINFOREQUEST._serialized_start=1436
  _GETLABELINDICESINFOREQUEST._serialized_end=1505
  _GETLABELINDICESINFORESPONSE._serialized_start=1508
  _GETLABELINDICESINFORESPONSE._serialized_end=1802
  _GETLABELINDICESINFORESPONSE_LABELINDEXINFO._serialized_start=1624
  _GETLABELINDICESINFORESPONSE_LABELINDEXINFO._serialized_end=1802
  _GETLABELINDICESINFORESPONSE_LABELINDEXINFO_LABELINDEXTYPE._serialized_start=1748
  _GETLABELINDICESINFORESPONSE_LABELINDEXINFO_LABELINDEXTYPE._serialized_end=1802
  _ADDLABELSREQUEST._serialized_start=1805
  _ADDLABELSREQUEST._serialized_end=2022
  _ADDLABELSRESPONSE._serialized_start=2024
  _ADDLABELSRESPONSE._serialized_end=2043
  _GETLABELSREQUEST._serialized_start=2045
  _GETLABELSREQUEST._serialized_end=2124
  _GENERICLABELS._serialized_start=2126
  _GENERICLABELS._serialized_end=2205
  _GENERICLABEL._serialized_start=2208
  _GENERICLABEL._serialized_end=2371
  _GETLABELSRESPONSE._serialized_start=2374
  _GETLABELSRESPONSE._serialized_end=2504
  _EVENTS._serialized_start=2507
  _EVENTS._serialized_end=4515
# @@protoc_insertion_point(module_scope)



# Copyright 2019 Regents of the University of Minnesota.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path

from mtap import GenericLabel
from mtap.io.brat import read_brat_documents


def test_read_brat_documents():
    d = Path(__file__).parent / 'brat'
    for event in read_brat_documents(d, document_name='the_gold',
                                     label_index_name_prefix='brat-',
                                     encoding='utf8'):
        doc = event.documents['the_gold']
        assert doc.document_name == 'the_gold'
        sentences = doc.labels['brat-Sentence']
        assert sentences == [
            GenericLabel(2, 26),
            GenericLabel(29, 67),
            GenericLabel(73, 97),
            GenericLabel(100, 138),
            GenericLabel(144, 217),
            GenericLabel(223, 269)
        ]
        unsures = doc.labels['brat-Unsure']
        assert unsures == [
            GenericLabel(2348, 2397)
        ]
